'''
This module contains a sciluigi subclass of luigi's Parameter, where
custom functionality might be added in the future.
'''

import luigi

class Parameter(luigi.Parameter):
    '''
    Subclass of luigi's Parameter, where custom functionality might be added in the future.
    '''
    pass
